@@ -1,3 +1,4 @@
+# encoding: utf-8
 " This sub-module provides 'sequence awareness' for blessed."
 
 __author__ = 'Jeff Quast <contact@jeffquast.com>'
@@ -5,12 +6,16 @@ __license__ = 'MIT'
 
 __all__ = ['init_sequence_patterns', 'Sequence', 'SequenceTextWrapper']
 
+# built-ins
 import functools
 import textwrap
 import warnings
 import math
 import re
 
+# 3rd-party
+import wcwidth  # https://github.com/jquast/wcwidth
+
 _BINTERM_UNSUPPORTED = ('kermit', 'avatar')
 _BINTERM_UNSUPPORTED_MSG = ('sequence-awareness for terminals emitting '
                             'binary-packed capabilities are not supported.')
@@ -58,59 +63,13 @@ def _build_any_numeric_capability(term, cap, num=99, nparams=1):
     return None  # no such capability
 
 
-def init_sequence_patterns(term):
-    """Given a Terminal instance, ``term``, this function processes
-    and parses several known terminal capabilities, and builds a
-    database of regular expressions and attaches them to ``term``
-    as attributes:
-
-    ``_re_will_move``
-      any sequence matching this pattern will cause the terminal
-      cursor to move (such as *term.home*).
-
-    ``_re_wont_move``
-      any sequence matching this pattern will not cause the cursor
-      to move (such as *term.bold*).
-
-    ``_re_cuf``
-      regular expression that matches term.cuf(N) (move N characters forward).
-
-    ``_cuf1``
-      *term.cuf1* sequence (cursor forward 1 character) as a static value.
-
-    ``_re_cub``
-      regular expression that matches term.cub(N) (move N characters backward).
-
-    ``_cub1``
-      *term.cuf1* sequence (cursor backward 1 character) as a static value.
-
-    These attributes make it possible to perform introspection on strings
-    containing sequences generated by this terminal, to determine the
-    printable length of a string.
-
-    For debugging, complimentary lists of these sequence matching pattern
-    values prior to compilation are attached as attributes ``_will_move``,
-    ``_wont_move``, ``_cuf``,  ``_cub``.
+def get_movement_sequence_patterns(term):
+    """ Build and return set of regexp for capabilities of ``term`` known
+        to cause movement.
     """
-    if term._kind in _BINTERM_UNSUPPORTED:
-        warnings.warn(_BINTERM_UNSUPPORTED_MSG)
-
-#    # for some reason, 'screen' does not offer hpa and vpa,
-#    # although they function perfectly fine !
-#    if term._kind == 'screen' and term.hpa == u'':
-#        def screen_hpa(*args):
-#            return u'\x1b[{}G'.format(len(args) and args[0] + 1 or 1)
-#        term.hpa = screen_hpa
-#    if term._kind == 'screen' and term.vpa == u'':
-#        def screen_vpa(*args):
-#            return u'\x1b[{}d'.format(len(args) and args[0] + 1 or 1)
-#        term.vpa = screen_vpa
-
     bnc = functools.partial(_build_numeric_capability, term)
-    bna = functools.partial(_build_any_numeric_capability, term)
-    # Build will_move, a list of terminal capabilities that have
-    # indeterminate effects on the terminal cursor position.
-    will_move_seqs = set([
+
+    return set([
         # carriage_return
         re.escape(term.cr),
         # column_address: Horizontal position, absolute
@@ -144,9 +103,15 @@ def init_sequence_patterns(term):
         term._cub,
     ])
 
-    # Build wont_move, a list of terminal capabilities that mainly affect
-    # video attributes, for use with measure_length().
-    wont_move_seqs = list([
+
+def get_wontmove_sequence_patterns(term):
+    """ Build and return set of regexp for capabilities of ``term`` known
+        not to cause any movement.
+    """
+    bnc = functools.partial(_build_numeric_capability, term)
+    bna = functools.partial(_build_any_numeric_capability, term)
+
+    return list([
         # print_screen: Print contents of screen
         re.escape(term.mc0),
         # prtr_off: Turn off printer
@@ -258,26 +223,79 @@ def init_sequence_patterns(term):
         # reset_{1,2,3}string: Reset string
     ] + map(re.escape, (term.r1, term.r2, term.r3,)))
 
-    # store pre-compiled list as '_will_move' and '_wont_move', for debugging
-    term._will_move = _merge_sequences(will_move_seqs)
-    term._wont_move = _merge_sequences(wont_move_seqs)
+
+def init_sequence_patterns(term):
+    """Given a Terminal instance, ``term``, this function processes
+    and parses several known terminal capabilities, and builds and
+    returns a dictionary database of regular expressions, which may
+    be re-attached to the terminal by attributes of the same key-name:
+
+    ``_re_will_move``
+      any sequence matching this pattern will cause the terminal
+      cursor to move (such as *term.home*).
+
+    ``_re_wont_move``
+      any sequence matching this pattern will not cause the cursor
+      to move (such as *term.bold*).
+
+    ``_re_cuf``
+      regular expression that matches term.cuf(N) (move N characters forward),
+      or None if temrinal is without cuf sequence.
+
+    ``_cuf1``
+      *term.cuf1* sequence (cursor forward 1 character) as a static value.
+
+    ``_re_cub``
+      regular expression that matches term.cub(N) (move N characters backward),
+      or None if terminal is without cub sequence.
+
+    ``_cub1``
+      *term.cuf1* sequence (cursor backward 1 character) as a static value.
+
+    These attributes make it possible to perform introspection on strings
+    containing sequences generated by this terminal, to determine the
+    printable length of a string.
+    """
+    if term._kind in _BINTERM_UNSUPPORTED:
+        warnings.warn(_BINTERM_UNSUPPORTED_MSG)
+
+    # Build will_move, a list of terminal capabilities that have
+    # indeterminate effects on the terminal cursor position.
+    _will_move = _merge_sequences(get_movement_sequence_patterns(term)
+                                  ) if term.does_styling else set()
+
+    # Build wont_move, a list of terminal capabilities that mainly affect
+    # video attributes, for use with measure_length().
+    _wont_move = _merge_sequences(get_wontmove_sequence_patterns(term)
+                                  ) if term.does_styling else set()
 
     # compile as regular expressions, OR'd.
-    term._re_will_move = re.compile('(%s)' % ('|'.join(term._will_move)))
-    term._re_wont_move = re.compile('(%s)' % ('|'.join(term._wont_move)))
+    _re_will_move = re.compile('(%s)' % ('|'.join(_will_move)))
+    _re_wont_move = re.compile('(%s)' % ('|'.join(_wont_move)))
+
+    # static pattern matching for horizontal_distance(ucs, term)
+    bnc = functools.partial(_build_numeric_capability, term)
 
-    # static pattern matching for _horiontal_distance
-    #
     # parm_right_cursor: Move #1 characters to the right
-    term._cuf = bnc(cap='cuf', optional=True)
-    term._re_cuf = re.compile(term._cuf) if term._cuf else None
+    _cuf = bnc(cap='cuf', optional=True)
+    _re_cuf = re.compile(_cuf) if _cuf else None
+
     # cursor_right: Non-destructive space (move right one space)
-    term._cuf1 = term.cuf1
+    _cuf1 = term.cuf1
+
     # parm_left_cursor: Move #1 characters to the left
-    term._cub = bnc(cap='cub', optional=True)
-    term._re_cub = re.compile(term._cub) if term._cub else None
+    _cub = bnc(cap='cub', optional=True)
+    _re_cub = re.compile(_cub) if _cub else None
+
     # cursor_left: Move left one space
-    term._cub1 = term.cub1
+    _cub1 = term.cub1
+
+    return {'_re_will_move': _re_will_move,
+            '_re_wont_move': _re_wont_move,
+            '_re_cuf': _re_cuf,
+            '_re_cub': _re_cub,
+            '_cuf1': _cuf1,
+            '_cub1': _cub1, }
 
 
 class SequenceTextWrapper(textwrap.TextWrapper):
@@ -290,7 +308,7 @@ class SequenceTextWrapper(textwrap.TextWrapper):
 
     def _wrap_chunks(self, chunks):
         """
-        escape-sequence aware varient of _wrap_chunks. Though
+        escape-sequence aware variant of _wrap_chunks. Though
         movement sequences, such as term.left() are certainly not
         honored, sequences such as term.bold() are, and are not
         broken mid-sequence.
@@ -387,57 +405,115 @@ class Sequence(unicode):
         terminal sequences.
 
         Although accounted for, strings containing sequences such as
-        ``term.clear`` will not give accurate returns, it is considered
-        un-lengthy (length of 0).
+        ``term.clear`` will not give accurate returns, it is not
+        considered lengthy (a length of 0). Combining characters,
+        are also not considered lengthy.
 
         Strings containing ``term.left`` or ``\b`` will cause "overstrike",
         but a length less than 0 is not ever returned. So ``_\b+`` is a
         length of 1 (``+``), but ``\b`` is simply a length of 0.
-        """
-        # TODO(jquast): Should we implement the terminal printable
-        # width of 'East Asian Fullwidth' and 'East Asian Wide' characters,
-        # which can take 2 cells, see http://www.unicode.org/reports/tr11/
-        # and http://www.gossamer-threads.com/lists/python/bugs/972834
-        return len(self.strip_seqs())
 
-    def strip(self):
-        """S.strip() -> unicode
+        Some characters may consume more than one cell, mainly those CJK
+        Unified Ideographs (Chinese, Japanese, Korean) defined by Unicode
+        as half or full-width characters.
 
-        Returns string derived from unicode string ``S``, stripped of
-        whitespace and terminal sequences.
+        For example:
+            >>> from blessed import Terminal
+            >>> from blessed.sequences import Sequence
+            >>> term = Terminal()
+            >>> Sequence(term.clear + term.red(u'コンニチハ')).length()
+            5
         """
-        return self.strip_seqs().strip()
+        # because combining characters may return -1, "clip" their length to 0.
+        clip = functools.partial(max, 0)
+        return sum(clip(wcwidth.wcwidth(w_char))
+                   for w_char in self.strip_seqs())
+
+    def strip(self, chars=None):
+        """S.strip([chars]) -> unicode
+
+        Return a copy of the string S with terminal sequences removed, and
+        leading and trailing whitespace removed.
+
+        If chars is given and not None, remove characters in chars instead.
+        """
+        return self.strip_seqs().strip(chars)
+
+    def lstrip(self, chars=None):
+        """S.lstrip([chars]) -> unicode
+
+        Return a copy of the string S with terminal sequences and leading
+        whitespace removed.
+
+        If chars is given and not None, remove characters in chars instead.
+        """
+        return self.strip_seqs().lstrip(chars)
+
+    def rstrip(self, chars=None):
+        """S.rstrip([chars]) -> unicode
+
+        Return a copy of the string S with terminal sequences and trailing
+        whitespace removed.
+
+        If chars is given and not None, remove characters in chars instead.
+        """
+        return self.strip_seqs().rstrip(chars)
 
     def strip_seqs(self):
         """S.strip_seqs() -> unicode
 
         Return a string without sequences for a string that contains
-        (most types) of (escape) sequences for the Terminal with which
-        they were created.
+        sequences for the Terminal with which they were created.
+
+        Where sequence ``move_right(n)`` is detected, it is replaced with
+        ``n * u' '``, and where ``move_left()`` or ``\\b`` is detected,
+        those last-most characters are destroyed.
+
+        All other sequences are simply removed. An example,
+            >>> from blessed import Terminal
+            >>> from blessed.sequences import Sequence
+            >>> term = Terminal()
+            >>> Sequence(term.clear + term.red(u'test')).strip_seqs()
+            u'test'
         """
         # nxt: points to first character beyond current escape sequence.
         # width: currently estimated display length.
+        input = self.padd()
+        outp = u''
+        nxt = 0
+        for idx in range(0, len(input)):
+            if idx == nxt:
+                # at sequence, point beyond it,
+                nxt = idx + measure_length(input[idx:], self._term)
+            if nxt <= idx:
+                # append non-sequence to outp,
+                outp += input[idx]
+                # point beyond next sequence, if any,
+                # otherwise point to next character
+                nxt = idx + measure_length(input[idx:], self._term) + 1
+        return outp
+
+    def padd(self):
+        """S.padd() -> unicode
+        Make non-destructive space or backspace into destructive ones.
+
+        Where sequence ``move_right(n)`` is detected, it is replaced with
+        ``n * u' '``.  Where sequence ``move_left(n)`` or ``\\b`` is
+        detected, those last-most characters are destroyed.
+        """
         outp = u''
         nxt = 0
         for idx in range(0, unicode.__len__(self)):
-            # account for width of sequences that contain padding (a sort of
-            # SGR-equivalent cheat for the python equivalent of ' '*N, for
-            # very large values of N that may xmit fewer bytes than many raw
-            # spaces. It should be noted, however, that this is a
-            # non-destructive space.
             width = horizontal_distance(self[idx:], self._term)
-            if width > 0:
-                outp += u' ' * horizontal_distance(self[idx:], self._term)
-            elif width < 0:
-                # \b causes the previous character to be trimmed
-                outp = outp[:width]
-            if idx == nxt:
-                # point beyond this sequence
+            if width != 0:
                 nxt = idx + measure_length(self[idx:], self._term)
+                if width > 0:
+                    outp += u' ' * width
+                elif width < 0:
+                    outp = outp[:width]
             if nxt <= idx:
                 outp += self[idx]
-                # point beyond next sequence, if any, otherwise next character
-                nxt = idx + measure_length(self[idx:], self._term) + 1
+                nxt = idx + 1
         return outp
 
 
@@ -523,11 +599,13 @@ def horizontal_distance(ucs, term):
         return -1
 
     elif ucs.startswith('\t'):
-        # as best as I can prove it, a tabstop is always 8 by default.
+        # As best as I can prove it, a tabstop is always 8 by default.
         # Though, given that blessings is:
+        #
         #  1. unaware of the output device's current cursor position, and
         #  2. unaware of the location the callee may chose to output any
         #     given string,
+        #
         # It is not possible to determine how many cells any particular
         # \t would consume on the output device!
         return 8
